;;; tamago-isearch-fix.el --- fix the i-search problem of Tamago-4
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Emacs-21 + Tamago-4 Զ륳
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(load "isearch-x")

;;; δؿΥꥸʥ mule-cmds.el ˤޤ
(defun read-multilingual-string-for-tamago (prompt &optional initial-input input-method)
  "Read a multilingual string from minibuffer, prompting with string PROMPT.
The input method selected last time is activated in minibuffer.
If optional second arg INITIAL-INPUT is non-nil, insert it in the minibuffer
initially.
Optional 3rd argument INPUT-METHOD specifies the input method
to be activated instead of the one selected last time.  It is a symbol
or a string."
  (setq input-method
	(or input-method
	    current-input-method
	    default-input-method
	    (read-input-method-name "Input method: " nil t)))
  (if (and input-method (symbolp input-method))
      (setq input-method (symbol-name input-method)))
  (let ((prev-input-method current-input-method))
    (unwind-protect
	(progn	;; Ѵ Ctrl+s Ǥ˥Ǥ褦ˤ롥
	  (setq map-for-tamago-isearch-multibyte-characters 
		(make-sparse-keymap))
	  (define-key map-for-tamago-isearch-multibyte-characters "\C-s" [13])
	  (define-key map-for-tamago-isearch-multibyte-characters "\C-r" [13])
	  (set-keymap-parent 
	   map-for-tamago-isearch-multibyte-characters 
	   minibuffer-local-map)
	  (activate-input-method input-method)
	  (read-from-minibuffer prompt initial-input 
				map-for-tamago-isearch-multibyte-characters
				nil nil nil t))
      (activate-input-method prev-input-method))))

;;; δؿΥꥸʥ isearch-x.el ˤޤ
;;;###autoload
(defun isearch-process-search-multibyte-characters-for-tamago (last-char)
  (if (eq this-command 'isearch-printing-char)
      (let ((overriding-terminal-local-map nil)
	    (prompt (concat (isearch-message-prefix) isearch-message))
	    (minibuffer-local-map isearch-minibuffer-local-map)
	    str)
	(if (not (equal
		  (substring 
		   (concat default-input-method "____________") 0 12)
		  "japanese-egg"))
	    (progn	;; Tamago-4 Ȥʤ
	      (if isearch-input-method-function
		  (let (;; Let input method work rather tersely.
			(input-method-verbose-flag nil))
		    (setq unread-command-events
			  (cons 'with-input-method
				(cons last-char unread-command-events))
			  ;; Inherit current-input-method in a minibuffer.
			  str (read-string prompt nil nil nil t))
		    (if (not str)
			;; All inputs were deleted while the input method
			;; was working.
			(setq str "")
		      (if (and (= (length str) 1)
			       (= (aref str 0) last-char)
			       (>= last-char 128))
			  ;; The input method couldn't handle LAST-CHAR.
			  (setq str nil)))))
	      (if (and (not str) (keyboard-coding-system))
		  (setq unread-command-events
			(cons 'with-keyboard-coding
			      (cons last-char unread-command-events))
			str (read-string prompt))))
	  (progn	;; Tamago-4 Ȥ 
	    (setq unread-command-events
		  (cons last-char unread-command-events))
	    (setq str (read-multilingual-string-for-tamago
		       (concat (isearch-message-prefix) isearch-message)
		       nil current-input-method))))
	
	(if (and str (> (length str) 0))
	    (let ((unread-command-events nil))
	      (isearch-process-search-string str str))
	  (isearch-update)))
    (isearch-process-search-char last-char)))

(defun isearch-printing-char ()
  "Add this ordinary printing character to the search string and search."
  (interactive)
  (let ((char last-command-char))
    (if (= char ?\S-\ )
        (setq char ?\ ))
    (if (and enable-multibyte-characters
             (>= char ?\200)
             (<= char ?\377))
        (if (keyboard-coding-system)
            (isearch-process-search-multibyte-characters-for-tamago char)
          (isearch-process-search-char (unibyte-char-to-multibyte char)))
      (if current-input-method
          (isearch-process-search-multibyte-characters-for-tamago char)
        (isearch-process-search-char char)))))

